const { ccclass, property } = cc._decorator;

@ccclass
export default class Test extends cc.Component {

    @property(cc.Node)
    bgNode: cc.Node = null;

    @property(cc.Node)
    particleNode: cc.Node = null;

    @property(cc.ProgressBar)
    progressBar: cc.ProgressBar = null;

    @property(cc.ProgressBar)
    progressBar2: cc.ProgressBar = null;

    private params: any;
    private progress = 0;
    

    start() {
        let node = this.progressBar.node;
        this.params = {};
        this.params.R = Math.sqrt(node.width / 2 * node.width / 2 + node.height / 2 * node.height / 2)
        this.params.x1 = node.position.x - node.width * node.anchorX;
        this.params.x2 = node.position.x + node.width * (1 - node.anchorX);
        this.params.y1 = node.position.y - node.height * node.anchorY;
        this.params.y2 = node.position.y + node.height * (1 - node.anchorY);
        this.params.width = node.width;
        this.params.height = node.height;
    }


    update(dt) {
        if (!this.params) return;
        let a = this.progress++ % 360;

        // 设置进度条
        let progress = a / 360;
        let radians = a * (Math.PI / 180);
        this.progressBar.progress = progress;
        this.progressBar2.progress = progress;

        let node = this.progressBar.node;
        // 斜边
        let AB = this.params.R;
        // 对边
        let AC = Math.sin(radians) * AB;
        // 临边
        let BC = Math.cos(radians) * AB;

        // 求出在四边形上的斜边长度AB后，求出粒子在四边形的坐标
        if (Math.abs(AC) >= this.params.height / 2) { // 将对应区域(上下两条边)坐标限制在四边形上，注释掉后对应区域会出现圆弧形轨迹
            AC = this.params.height / 2;
            AB = AC / Math.sin(radians);
        }
        if (Math.abs(BC) >= this.params.width / 2) { // 将对应区域(左右两条边)坐标限制在四边形上，注释掉后对应区域会出现圆弧形轨迹
            BC = this.params.width / 2;
            AB = BC / Math.cos(radians);
        }
        AB = Math.abs(AB);

        // 求出 x y 坐标
        let x = node.position.x + AB * Math.cos(radians);
        let y = node.position.y + AB * Math.sin(radians);

        // 设置粒子坐标
        this.particleNode.position = cc.v2(x, y);
    }
}
